//Copyright (c) 2007 Oak Circle Software, Inc.
//
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.

#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <string>
#include <iostream>

using namespace boost::posix_time;
using namespace boost::gregorian;

using std::cin;
using std::cout;
using std::endl;
using std::string;

static ptime parseDate(const string& directoryName) {
	// The directoryName variable will contain a directory name, in the format
	// "2007-07-02_14.57.17.558834.ubuntu.ful". We need to parse the date and
	// time from it.
	boost::regex expr("(\\d{4}-\\d{2}-\\d{2})_(\\d{2}\\.\\d{2}\\.\\d{2}\\.\\d+)\\.(.+).(ful|inc)");
	boost::smatch m;
	if (boost::regex_match(directoryName, m, expr)) {
		// If it's a valid directory name, m[1].str() will return the date,
		// m[2].str() will return the time, and m[3].str() will return the
		// computer name.
		return ptime(time_from_string(m[1].str()+' '+m[2].str()));
	} else {
		// Otherwise it isn't a valid date/time directory name. 
		return ptime(not_a_date_time);
	};
};

static time_duration ageOf(const string& directoryName) {
	ptime t=parseDate(directoryName);
	ptime now=second_clock::local_time();
	if (!t.is_not_a_date_time() && t <= now) {
		return time_period(t, now).length();
	} else {
		// This isn't allowed. We need to ignore it.
		return time_duration(not_a_date_time);
	};
};

int main(int argc, char *argv[]) {
	// Read the directory names from stdin, one per line.
	int count=0;
	time_duration newest(not_a_date_time);
	string directoryName;
	while (std::getline(cin, directoryName)) {
		time_duration age=ageOf(directoryName);
		
		//cout << directoryName << ": " << age.hours() << " hours old" << endl;
		
		if (!age.is_not_a_date_time()) {
			++count;
			if (newest.is_not_a_date_time() || age < newest) {
				//cout << "Setting newest..." << endl;
				newest=age;
			};
		};
	};
	
	if (count==0) {
		cout << "No filenames in input data. Server crashed?" << endl;
	} else if (newest.hours() < 24) {
		cout << "Most recent backup is " << newest.hours() << " hours old." << endl;
	} else {
		cout << "ERROR! Most recent backup is " << newest.hours() << " hours old!" << endl;
	};
	
	// TODO: Only for the machine we're supposed to check for?
	
	return 0;
};
